package cn.com.tenlion.bigdata.service.bigdatamodule;

import cn.com.tenlion.bigdata.pojo.dtos.bigdatamodule.BigdataModuleDTO;
import cn.com.tenlion.bigdata.pojo.vos.bigdatamodule.BigdataModuleVO;
import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultData;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: IBigdataModuleService
 * @Description: 大数据展示模块
 * @Author: WenG
 * @Date: 2020-10-19 16:18
 * @Version: 1.0
 **/
public interface IBigdataModuleService {

    /**
     * 新增大数据展示模块
     *
     * @param bigdataModuleVO
     * @return
     * @throws Exception
     */
    SuccessResult saveBigdataModule(BigdataModuleVO bigdataModuleVO) throws Exception;

    /**
     * 新增大数据展示模块(APP)
     *
     * @param token
     * @param bigdataModuleVO
     * @return
     * @throws Exception
     */
    SuccessResult saveBigdataModuleByToken(String token, BigdataModuleVO bigdataModuleVO) throws Exception;

    /**
     * 新增大数据展示模块
     *
     * @param bigdataModuleVO
     * @return bigdataModuleId
     * @throws Exception
     */
    String saveBigdataModuleReturnId(BigdataModuleVO bigdataModuleVO) throws Exception;

    /**
     * 新增大数据展示模块(APP)
     *
     * @param token
     * @param bigdataModuleVO
     * @return bigdataModuleId
     * @throws Exception
     */
    String saveBigdataModuleByTokenReturnId(String token, BigdataModuleVO bigdataModuleVO) throws Exception;

    /**
     * 删除大数据展示模块
     *
     * @param ids
     * @return
     * @throws RemoveException
     */
    SuccessResult removeBigdataModule(String ids) throws RemoveException;

    /**
     * 删除大数据展示模块（物理删除）
     *
     * @param ids
     * @throws RemoveException
     */
    void deleteBigdataModule(String ids) throws RemoveException;

    /**
     * 删除大数据展示模块(APP)
     *
     * @param token
     * @param ids
     * @return
     * @throws RemoveException
     */
    SuccessResult removeBigdataModuleByToken(String token, String ids) throws RemoveException;

    /**
     * 修改大数据展示模块
     *
     * @param bigdataModuleId
     * @param bigdataModuleVO
     * @return
     * @throws Exception
     */
    SuccessResult updateBigdataModule(String bigdataModuleId, BigdataModuleVO bigdataModuleVO) throws Exception;

    /**
     * 修改大数据展示模块(APP)
     *
     * @param token
     * @param bigdataModuleId
     * @param bigdataModuleVO
     * @return
     * @throws Exception
     */
    SuccessResult updateBigdataModuleByToken(String token, String bigdataModuleId, BigdataModuleVO bigdataModuleVO) throws Exception;

    /**
     * 大数据展示模块详情(通过ID)
     *
     * @param bigdataModuleId
     * @return
     * @throws SearchException
     */
    BigdataModuleDTO getBigdataModuleById(String bigdataModuleId) throws SearchException;

    /**
     * 大数据展示模块列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<BigdataModuleDTO> listBigdataModule(Map<String, Object> params) throws SearchException;

    /**
     * 大数据展示模块分页列表
     *
     * @param page
     * @return
     * @throws SearchException
     */
    SuccessResultList<List<BigdataModuleDTO>> listPageBigdataModule(ListPage page) throws SearchException;

    /**
     * 大数据展示模块统计
     *
     * @param params
     * @return
     * @throws SearchException
     */
    Integer countNumberBigdataModule(Map<String, Object> params) throws SearchException;

    /**
     * 大数据展示模块统计
     *
     * @param params
     * @return
     * @throws SearchException
     */
    SuccessResultData<Integer> countBigdataModule(Map<String, Object> params) throws SearchException;

    SuccessResult updateBigdataModuleContent(String bigdataModuleId, String bigdataModuleContent) throws Exception;

    SuccessResult updateBigdataModuleSwitch(String bigdataModuleId, Boolean bigdataModuleSwitch) throws Exception;

    SuccessResult buiderBigdataTemplateAll(String bigdataModuleId) throws Exception;
}